
#define myVersionNumber 3.7

/*
 *	SAC IO functions for ANSI C
 *
 *      Version 3.7 - CJA / Penn State November 2003
 *        added IsSacFile function
 *
 *      Version 3.6 - CJA / Penn State August 2003
 *        added fclearstring_ to clear fortran strings before use.
 *
 *      Version 3.5 - CJA / Penn State  August 2003
 *         changed WriteSac2 ReadSac2
 *
 *      Version 3.4 - CJA / Penn State  July 2003
 *         Added inihdr_ fortran stub that does nothing but save porting time.
 *         setkhv do nothign stub - again to save editing...
 *
 *      Version 3.3 - CJA / Penn State  July 2003
 *         Added rsac2 for Jordi - not tested.
 *
 *      Version 3.2 - CJA / Penn State  July 2003
 *         Fixed a write-success test in WriteSAC2.
 *	   Added PrintSacIOVersion function
 *
 *      Version 3.1 - CJA / Penn State  May 2003
 *         Created wsac2 fortran stub.
 *         
 *      Version 3.0 - CJA / Penn State  May 2003
 *         Started using custom routines for translating fortran strings.
 *         Part of a long struggle with fortran string nonsense.
 *
 *      Version 2.5 - CJA / Penn State
 *        removed more strtok calls
 *
 *      Version 2.4 - CJA / Penn State
 *      Changes the way the fortran stubs compared the header-field
 *         strings so that the call worked with g77 on a mac
 *
 *	Version 2.3 - Chuck Ammon Penn State
 *	Changes made to sacio.h to set some default values
 *	for lovrok,lcalda,iftype,idep
 *      added magnitude fields and database internal fields.
 *
 *	Version 2.2 - Chuck Ammon Saint Louis University
 *
 *      March, 2000
 *
 *	I wanted to process SAC files on the my MAC & Fortran is too expensive
 *		on the MAC.
 *
 *	This is a modification of a set of IO routines I got from George Randall
 *		and are based on earlier functions written by Lupei Zhu
 *
 *
 *	To use the C functions:
 *  ----------------------------------
 *
 *		you need the files: sacio.c and sacio.h
 *
 *		If you intend to manipulate the headers, you must include sacio.h
 *		in the c-code that does the manipulating since the sac file header
 *		is stored in a structure defined in sac.h. It's much more convenient
 *		than the old getfhv()... in FORTRAN.
 *
 *		If you are setting a lot of header values, use ReadSACfile
 *		and WriteSACfile.
 *
 *		If you are doing bare bones headers, like wsac1 and wsac2 in FORTRAN,
 *			just use: WriteSAC1, WriteSAC2
 *
 *	To use the FORTRAN stubs:
 *  ----------------------------------
 *
 *              The header access functions are incomplete, although most 
 *                common header values are included in the [g,s]et[f,n,l]hv
 *                functions. I did not write a complete [g,s]etkhv routine. These  
 *                are what I needed right now to get some of my codes running on 
 *                machines where sac is not available.
 *
 *                To use these functions, make the library and then link your
 *                 FORTRAN code with this lib in place of the sac library.
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sacio.h"

/* Function PROTOTYPES */
void printSacIOVersion();
void NewSacHeader(struct SacHeader *hd);
void ReadSACHeader(char *filename, struct SacHeader *header, int *io_error);
void ReadSACfile(char *filename,float *x,struct SacHeader *header,int maxpts,int *io_error);
void ReadSAC1(char *filename,float *x,int *npts,float *dt,float *b,int maxpts,int *io_error);
void ReadSAC2(char *filename, int *n, float *x, float *y, int maxpts, int *io_error);
void WriteSACfile(char *filename,float *x,struct SacHeader header,int *io_error);
void WriteSAC1(char *filename,float *x,int npts,float dt,float b,int *io_error);
void WriteSAC2(char *filename,int n,float *x,float *y,int *io_error);\
int  IsSacFile(char *filename);

/* for fortran or f2c calls */

void wsac1_(char *filename,float *x,int *npts,float *b,float *dt,int *io_error,long slen);
void rsac1_(char *filename,float *x,int *npts,float *b,float *dt,int *maxpts,int *io_error, long slen);
void wsac0_(char *filename,float *x,float *y, int *io_error, long slen);
void wsac2_(char *filename,float *y, int *npts, float *x, int *io_error, long slen);
void newhdr_();
void getfhv_(char *label, float *value, int *access_error, long slen);
void setfhv_(char *label, float *value, int *access_error, long slen);
void getnhv_(char *label, int *value, int *access_error, long slen);
void setnhv_(char *label, int *value, int *access_error, long slen);
void setihv_(char *label, char *value, int *access_error, long slen, long vlen);
void setlhv_(char *label, int *value, int *access_error, long slen);
void getkhv_(char *label, char *value, int *access_error, long slen, long vlen);
void setkhv_(char *label, char *value, int *access_error, long slen, long vlen);
void inihdr_();

int fslen(char *filename,long slen);
void fscpy(char *cc, char *fs, int slen);
void fclearstring_(char *s, int slen);

/*****************************************************************************/

void printSacIOVersion()
{
    printf("iSacIO library version: %s\n", "myVersionNumber");
}

/*****************************************************************************/
/*
 * FUNCTION TO TEST IF A FILE IS AN EVENLY_SPACED SAC FILE
 * 
 */
/*****************************************************************************/
 
int IsSacFile(char *filename)
{
	FILE           *strm;
	int             hsize, fsize, nread, yes=1, no=-1;
	struct SacHeader	h;
	struct stat sb;

	hsize = sizeof(struct SacHeader);
	/* first check for the file existence, and get the file size */
	
	if(stat(filename, &sb) == -1) return(no);
	fsize = (int) sb.st_size;
	
	if(fsize < hsize) return(no);
	
	/* open the stream */
	strm = fopen(filename, "rb");
	if (strm == NULL) {
		return(no);
	}
	
	/* read in the file header */
	nread = fread(&h, sizeof(struct SacHeader), 1, strm);
	if (nread != 1) {
		fclose(strm);
		return(no);
	}
	fclose(strm);
	
	/* apply some validity tests */
	if(h.delta <= 0.0) return(no);
	if(h.npts  <= 0  ) return(no);
	if(h.nvhdr <  0  ) return(no);
	if( (hsize + h.npts*sizeof(float)) < fsize) return(no);

	/* passed all the tests */
	return(yes);
}

/*****************************************************************************/

void NewSacHeader(struct SacHeader *hd)
{
	*hd = sac_null;
	hd->nvhdr = 6;
	hd->norid = 0;
	hd->nevid = 0;
	hd->lpspol = FALSE;
	hd->lcalda = TRUE;
	hd->unused27 = FALSE;
}

/*****************************************************************************/
/*
 * FUNCTION TO READ IN A SAC HEADER
 * 
 * read in a sac file header filename is the name of the SAC file header is the
 */
/*****************************************************************************/

void ReadSACHeader(char *filename, struct SacHeader *header, int *io_error)
{
	FILE           *strm;
	int             nread;

	*io_error = kNO_ERROR;

	/* open the stream */
	strm = fopen(filename, "rb");
	if (strm == NULL) {
		*io_error = kIO_ERROR;
		return;
	}
	
	/* read in the header values */
	nread = fread(header, sizeof(struct SacHeader), 1, strm);
	
	fclose(strm);

	if (nread != 1)
		*io_error = kIO_ERROR;
}

/*****************************************************************************/
/*
 * FUNCTION TO READ IN AN EVENLY_SPACED SAC FILE
 * 
 *	filename is the name of the SAC file header
 *	x is the array for the time samples
 *	header is the sac header structure
 *	maxpts is the max number of points to input
 * 
 */
/*****************************************************************************/

void
ReadSACfile(char *filename,float *x,struct SacHeader *header,int maxpts, int *io_error)
{
	FILE           *strm;
	int             nread;

	*io_error = kNO_ERROR;

	/* open the stream */
	strm = fopen(filename, "rb");
	if (strm == NULL) {
		*io_error = kIO_ERROR;
		return;
	}
	/* read in the file header */

	nread = fread(header, sizeof(struct SacHeader), 1, strm);
	if (nread != 1) {
		*io_error = kIO_ERROR;
		fclose(strm);
		return;
	}
	
	/* check to make sure we don't overfill memory */
	if (header->npts > maxpts)
	{
		header->npts = maxpts;
	}
	
	/* read in the floating point amplitudes */

	nread = fread(x, header->npts * sizeof(float), 1, strm);

	fclose(strm);
	if (nread != 1)
		*io_error = kIO_ERROR;
}
/*****************************************************************************/
/*
 * FUNCTION TO READ IN AN EVENLY_SPACED SAC FILE
 * 
 * Assuming that you don't want header details
 *
 * 	filename is the name of the SAC file 
 * 	x is the array into which the data are read
 *	npts = number of points read in (<= maxpts)
 *	dt is the sample rate
 *	b is the time of the first sample
 */
/*****************************************************************************/

void
ReadSAC1(char *filename,float *x,int *npts,float *dt,float *b,int maxpts,int *io_error)
{
	struct SacHeader	hd;

	ReadSACfile(filename,x,&hd,maxpts,io_error);
	
	if(*io_error == kIO_ERROR)
	{
		return;
	}	

	*npts = hd.npts;
	*dt = hd.delta;
	*b =  hd.b;

}
/*****************************************************************************/
/*
 * FUNCTION TO READ AN UNEVENLY_SPACED SAC FILE
 * 
 *	filename is the name of the SAC file
 *	n is the number of data points read in
 *	x and y are the data read in
 *	maxpts is the maximum number of data to read in (dimensions of x and y)
 * 
 */
/*****************************************************************************/
void 
ReadSAC2(char *filename, int *n, float *x, float *y, int maxpts, int *io_error)
{
	FILE *strm;
	struct SacHeader hd;
	void NewSacHeader();
	int nread;
	
	/* OPEN THE stream for writing the file */

	*io_error = kNO_ERROR;

	/* open the stream */
	strm = fopen(filename, "rb");
	if (strm == NULL) {
		*io_error = kIO_ERROR;
		return;
	}	
	
	/* Now read the sac header to the file */
	nread = fread(&hd, sizeof(struct SacHeader), 1, strm);
		if (nread != 1) {
		*io_error = kIO_ERROR;
		fclose(strm);
		return;
	}
	
	/* set up the number of points in the file, and apply the limits */
	*n = hd.npts;
	if(*n > maxpts) *n = maxpts;
	/* Now write the sac data points x, then y, from the file */
	nread = fread(y, sizeof(float), *n, strm);
		if (nread != *n) {
		*io_error = kIO_ERROR;
		fclose(strm);
		return;
	}
	nread = fread(x, sizeof(float), *n, strm);
		if (nread != *n) {
		*io_error = kIO_ERROR;
		fclose(strm);
		return;
	}
	
	fclose(strm);
}

/*****************************************************************************/
/*
 * FUNCTION TO WRITE AN EVENLY_SPACED SAC FILE
 * 
 * Assuming that the header is all set up
 * 
 * filename is the name of the SAC file header is the file header ar is the
 * array into which the data are read 
 */
/*****************************************************************************/

void
WriteSACfile(char *filename,float *x,struct SacHeader header,int *io_error)
{
	FILE           *strm;
	int             nwritten;

	/* V2.2 03/00 make sure we have a consistent start and end */
	header.e = header.b + (header.npts - 1) * header.delta;
	
	*io_error = kNO_ERROR;

	/* open the stream */
	strm = fopen(filename, "w");
	if (strm == NULL) {
		*io_error = kIO_ERROR;
		return;
	}
	/* write in the file header */

	nwritten = fwrite(&header, sizeof(struct SacHeader), 1, strm);
	if (nwritten != 1) {
		*io_error = kIO_ERROR;
		fclose(strm);
		return;
	}
	/* read in the floating point amplitudes */

	nwritten = fwrite(x, header.npts * sizeof(float), 1, strm);

	fclose(strm);

	if (nwritten != 1)
		*io_error = kIO_ERROR;
}

/*****************************************************************************/
/*
 * FUNCTION TO WRITE AN EVENLY_SPACED SAC FILE
 * 
 * Assuming that the header is NOT set up ---
 * 
 * filename is the name of the SAC file, x is the
 * array from which the data are written,npts is the number of pts,
 * dt is the sample increment, b is the time fo the first sample (secs)
 * 
 */
/*****************************************************************************/

void
WriteSAC1(char *filename,float *x,int npts,float dt,float b,int *io_error)
{
	struct SacHeader	hd;
	void        	    NewSacHeader();

	/* set up a minimum SAC header for the output file */

	NewSacHeader(&hd);

	hd.npts = npts;
	hd.iftype = ITIME;
	hd.delta = dt;
	hd.b = b;
	hd.e = b + (hd.npts - 1) * hd.delta;
	hd.iztype = IB;
	hd.leven = TRUE;

	WriteSACfile(filename,x,hd,io_error);

}

/*****************************************************************************/
/*
 * FUNCTION TO WRITE AN UNEVENLY_SPACED SAC FILE
 * 
 * Assuming that the header is NOT set up ---
 * 
 * filename is the name of the SAC file, x and y are the
 * arrays from which the data are written,
 */
/*****************************************************************************/
void
WriteSAC2(char *filename, int n, float *x, float *y, int *io_error)
{
	FILE *strm;
	struct SacHeader hd;
	void NewSacHeader();
	int nwritten;
	
	/* construct a minimum header */
	NewSacHeader(&hd);
	hd.npts = n;
	hd.iftype = IXY;
	hd.leven = FALSE;
	
	/* OPEN THE stream for writing the file */

	*io_error = kNO_ERROR;

	/* open the stream */
	strm = fopen(filename, "w");
        if (strm == NULL) {
            *io_error = kIO_ERROR;
            printf("Problem creating file %s\n",filename);
            return;
	}	
	
	/* Now write the sac header to the file */
	nwritten = fwrite(&hd, sizeof(struct SacHeader), 1, strm);
		if (nwritten != 1) {
		*io_error = kIO_ERROR;
		fclose(strm);
                printf("Problem writing header for file %s\n",filename);
		return;
	}
	/* Now write the sac data points x, then y, to the file */
	nwritten = fwrite(y, sizeof(float), n, strm);
		if (nwritten != n) {
		*io_error = kIO_ERROR;
		fclose(strm);
                printf("Problem writing x values for file %s\n",filename);
		return;
	}
	nwritten = fwrite(x, sizeof(float), n, strm);
		if (nwritten != n) {
		*io_error = kIO_ERROR;
		fclose(strm);
                printf("Problem writing y values for file %s\n",filename);
		return;
	}
	
	fclose(strm);
}

/*****************************************************************************/
/*
 * FORTRAN Stubs
 * 
 */
/*****************************************************************************/

/* this is a global value used for header access in fortran */

struct SacHeader ghdr;

void newhdr_()
{
	ghdr = sac_null;
}

void inihdr_()
{
    ghdr = sac_null;
}

void wsac1_(char *filename,float *x,int *npts,float *b,float *dt,int *io_error, long slen)
{

	char token[512];

	fscpy(token,filename,slen);
		
	WriteSAC1(token, x, *npts, *dt, *b, io_error);

}

void wsac2_(char *filename,float *y, int *npts, float *x, int *io_error, long slen)
{

    char token[512];

    fscpy(token,filename,slen);

    WriteSAC2(token,*npts,x,y,io_error);
}


void wsac0_(char *filename,float *x,float *y, int *io_error, long slen)
{
	char token[512];

	fscpy(token,filename,slen);
	
	/* write out the values in the global header, ghdr */
		
	/* ANYTHING you want in the header had better be 
	 * SET BEFORE calling this function.
	 * This includes the number of points, delta, etc.
	*/
	if(ghdr.nvhdr == -12345) ghdr.nvhdr = 6;
	WriteSACfile(token, y, ghdr, io_error);

}
void rsac1_(char *filename,float *x,int *npts,float *b,float *dt,int *maxpts,int *io_error,long slen)
{
	char token[512];

	fscpy(token,filename,slen);
	
	ReadSACHeader(token, &ghdr, io_error);
	if(*io_error == kIO_ERROR)
	  printf("Problem reading the SAC Header (io_error = %d, name = %s)\n",*io_error,token);
	ReadSAC1(token,x,npts,dt,b,*maxpts,io_error);
	if(*io_error == kIO_ERROR)
	  printf("Problem reading the SAC File (io_error = %d, name = %s)\n",*io_error,token);
	
}
void rsac2_(char *filename,float *y,int *npts,float *x,int *maxpts,int *io_error,long slen)
{
    char token[512];

    fscpy(token,filename,slen);

    ReadSACHeader(token, &ghdr, io_error);
    if(*io_error == kIO_ERROR)
        printf("Problem reading the SAC XY Header (io_error = %d, name = %s)\n",*io_error,token);
    ReadSAC2(token,npts,x,y,*maxpts,io_error);
    if(*io_error == kIO_ERROR)
        printf("Problem reading the SAC XY File (io_error = %d, name = %s)\n",*io_error,token);

}
void getfhv_(char *label, float *value, int *access_error, long slen)
{	
	char token[16];
	char thelabel[16];
	int i;
	
	token[0] = '\0';
	strncpy(token,label,(size_t) slen);
	/* convert to upper case */
	for(i=0;i<slen;i++)
	{
	    thelabel[i] = toupper((int) token[i]);
	}
	thelabel[i] = '\0';
	
	/* printf("thelabel  = %s\n",thelabel);*/
	
	if(strcmp(thelabel,"B") == 0)
	{
	     *value = ghdr.b;
	      return;
	}
	else if(strcmp(thelabel,"E") == 0)
	{
	     *value = ghdr.e;
	      return;
	}
	else if(strcmp(thelabel,"DELTA") == 0)
	{
	     *value = ghdr.delta;
	      return;
	}
	else if(strcmp(thelabel,"DEPMIN") == 0)
	{
	     *value = ghdr.depmin;
	      return;
	}
	else if(strcmp(thelabel,"DEPMEN") == 0)
	{
	     *value = ghdr.depmen;
	      return;
	}
	else if(strcmp(thelabel,"DEPMAX") == 0)
	{
	     *value = ghdr.depmax;
	      return;
	}
	else if(strcmp(thelabel,"EVLA") == 0)
	{
	     *value = ghdr.evla;
	      return;
	}
	else if(strcmp(thelabel,"EVLO") == 0)
	{
	     *value = ghdr.evlo;
	      return;
	}
	else if(strcmp(thelabel,"EVDP") == 0)
	{
	     *value = ghdr.evdp;
	      return;
	}
	else if(strcmp(thelabel,"STLA") == 0)
	{
	     *value = ghdr.stla;
	      return;
	}
	else if(strcmp(thelabel,"STLO") == 0)
	{
	     *value = ghdr.stlo;
	      return;
	}
	else if(strcmp(thelabel,"STEL") == 0)
	{
	     *value = ghdr.stel;
	      return;
	}
	else if(strcmp(thelabel,"CMPAZ") == 0)
	{
	     *value = ghdr.cmpaz;
	      return;
	}
	else if(strcmp(thelabel,"CMPINC") == 0)
	{
	     *value = ghdr.cmpinc;
	      return;
	}
	else if(strcmp(thelabel,"DIST") == 0)
	{
	     *value = ghdr.dist;
	      return;
	}
	else if(strcmp(thelabel,"GCARC") == 0)
	{
	     *value = ghdr.gcarc;
	      return;
	}
	else if(strcmp(thelabel,"AZ") == 0)
	{
	     *value = ghdr.az;
	      return;
	}
	else if(strcmp(thelabel,"BAZ") == 0)
	{
	     *value = ghdr.baz;
	      return;
	}
	else if(strcmp(thelabel,"O") == 0)
	{
	     *value = ghdr.o;
	      return;
	}
	else if(strcmp(thelabel,"A") == 0)
	{
	     *value = ghdr.a;
	      return;
	}
	else if(strcmp(thelabel,"T0") == 0)
	{
	     *value = ghdr.t0;
	      return;
	}
	else if(strcmp(thelabel,"T1") == 0)
	{
	     *value = ghdr.t1;
	      return;
	}
	else if(strcmp(thelabel,"T2") == 0)
	{
	     *value = ghdr.t2;
	      return;
	}
	else if(strcmp(thelabel,"T3") == 0)
	{
	     *value = ghdr.t3;
	      return;
	}
	else if(strcmp(thelabel,"T4") == 0)
	{
	     *value = ghdr.t4;
	      return;
	}
	else if(strcmp(thelabel,"T5") == 0)
	{
	     *value = ghdr.t5;
	      return;
	}
	else if(strcmp(thelabel,"T6") == 0)
	{
	     *value = ghdr.t6;
	      return;
	}
	else if(strcmp(thelabel,"T7") == 0)
	{
	     *value = ghdr.t7;
	      return;
	}
	else if(strcmp(thelabel,"T8") == 0)
	{
	     *value = ghdr.t8;
	      return;
	}
	else if(strcmp(thelabel,"T9") == 0)
	{
	     *value = ghdr.t9;
	      return;
	}
	else if(strcmp(thelabel,"USER0") == 0)
	{
	     *value = ghdr.user0;
	      return;
	}
	else if(strcmp(thelabel,"USER1") == 0)
	{
	     *value = ghdr.user1;
	      return;
	}
	else if(strcmp(thelabel,"USER2") == 0)
	{
	     *value = ghdr.user2;
	      return;
	}
	else if(strcmp(thelabel,"USER3") == 0)
	{
	     *value = ghdr.user3;
	      return;
	}
	else if(strcmp(thelabel,"USER4") == 0)
	{
	     *value = ghdr.user4;
	      return;
	}
	else if(strcmp(thelabel,"USER5") == 0)
	{
	     *value = ghdr.user5;
	      return;
	}
	else if(strcmp(thelabel,"USER6") == 0)
	{
	     *value = ghdr.user6;
	      return;
	}
	else if(strcmp(thelabel,"USER7") == 0)
	{
	     *value = ghdr.user7;
	      return;
	}
	else if(strcmp(thelabel,"USER8") == 0)
	{
	     *value = ghdr.user8;
	      return;
	}
	else if(strcmp(thelabel,"USER9") == 0)
	{
	     *value = ghdr.user9;
	      return;
	}
	else
	{
	       *value = -12345;
	       printf("Warning %s not matched in getfhv\n",thelabel);
	       return;
	}	

}
void getkhv_(char *label, char *value, int *access_error, long slen, long vlen)
{	
	char token[16];
	char thelabel[16];
	int i;
	
	strncpy(token,label,(size_t) slen);
	/* convert to upper case */
	for(i=0;i<slen;i++)
	{
	    thelabel[i] = toupper((int) token[i]);
	}
	thelabel[i] = '\0';
	
	/*printf("token = %s thelabel = %s",token,thelabel);*/
	
	if(strcmp(thelabel,"KCMPNM") == 0)
	{
	      strncpy(value,ghdr.kcmpnm,(size_t) 8);
	      return;
	}
	else if(strcmp(thelabel,"KSTNM") == 0)
	{
	      strncpy(value,ghdr.kstnm,(size_t) 8);
	      return;
	}
	else
	{
	       strncpy(value,"-12345",(size_t) 6);
	       fprintf(stderr,"Warning %s not matched in getfhv\n",thelabel);
	       return;
	}	

}

void setkhv_(char *label, char *value, int *access_error, long slen, long vlen)
{
    fprintf(stderr,"Warning setkhv is not implemented yet.\n");
}

void setfhv_(char *label, float *value, int *access_error, long slen)
{	
	char token[16];
	char thelabel[16];
	int i;
	
	thelabel[0] = '\0';
           token[0] = '\0';

	strncpy(token,label,(size_t) slen);
	/* convert to upper case */
	for(i=0;i<slen;i++)
	{
	    thelabel[i] = toupper((int) token[i]);
	}
	thelabel[i] = '\0';
	
	/*printf("token = %s thelabel = %s\n",token,thelabel);*/
	
	if(strcmp(thelabel,"B") == 0)
	{
	     ghdr.b = *value ;
	      return;
	}
	else if(strcmp(thelabel,"E") == 0)
	{
	     ghdr.e = *value;
	      return;
	}
	else if(strcmp(thelabel,"DELTA") == 0)
	{
	     ghdr.delta = *value;
	      return;
	}
	else if(strcmp(thelabel,"DEPMIN") == 0)
	{
	     ghdr.depmin = *value;
	      return;
	}
	else if(strcmp(thelabel,"DEPMEN") == 0)
	{
	     ghdr.depmen = *value;
	      return;
	}
	else if(strcmp(thelabel,"DEPMAX") == 0)
	{
	     ghdr.depmax = *value;
	      return;
	}
	else if(strcmp(thelabel,"EVLA") == 0)
	{
	      ghdr.evla = *value;
	      return;
	}
	else if(strcmp(thelabel,"EVLO") == 0)
	{
	     ghdr.evlo = *value;
	      return;
	}
	else if(strcmp(thelabel,"EVDP") == 0)
	{
	     ghdr.evdp = *value;
	      return;
	}
	else if(strcmp(thelabel,"STLA") == 0)
	{
	     ghdr.stla = *value;
	      return;
	}
	else if(strcmp(thelabel,"STLO") == 0)
	{
	     ghdr.stlo = *value;
	      return;
	}
	else if(strcmp(thelabel,"STEL") == 0)
	{
	     ghdr.stel = *value;
	      return;
	}
	else if(strcmp(thelabel,"CMPAZ") == 0)
	{
	     ghdr.cmpaz = *value;
	      return;
	}
	else if(strcmp(thelabel,"CMPINC") == 0)
	{
	     ghdr.cmpinc = *value;
	      return;
	}
	else if(strcmp(thelabel,"DIST") == 0)
	{
	     ghdr.dist = *value;
	      return;
	}
	else if(strcmp(thelabel,"GCARC") == 0)
	{
	     ghdr.gcarc = *value;
	      return;
	}
	else if(strcmp(thelabel,"AZ") == 0)
	{
	     ghdr.az = *value;
	      return;
	}
	else if(strcmp(thelabel,"BAZ") == 0)
	{
	     ghdr.baz = *value;
	      return;
	}
	else if(strcmp(thelabel,"O") == 0)
	{
	     ghdr.o = *value;
	      return;
	}
	else if(strcmp(thelabel,"A") == 0)
	{
	     ghdr.a = *value;
	      return;
	}
	else if(strcmp(thelabel,"T0") == 0)
	{
	     ghdr.t0 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T1") == 0)
	{
	     ghdr.t1 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T2") == 0)
	{
	     ghdr.t2 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T3") == 0)
	{
	     ghdr.t3 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T4") == 0)
	{
	     ghdr.t4 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T5") == 0)
	{
	     ghdr.t5 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T6") == 0)
	{
	     ghdr.t6 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T7") == 0)
	{
	     ghdr.t7 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T8") == 0)
	{
	     ghdr.t8 = *value;
	      return;
	}
	else if(strcmp(thelabel,"T9") == 0)
	{
	     ghdr.t9 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER0") == 0)
	{
	     ghdr.user0 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER1") == 0)
	{
	     ghdr.user1 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER2") == 0)
	{
	     ghdr.user2 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER3") == 0)
	{
	     ghdr.user3 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER4") == 0)
	{
	     ghdr.user4 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER5") == 0)
	{
	     ghdr.user5 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER6") == 0)
	{
	     ghdr.user6 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER7") == 0)
	{
	     ghdr.user7 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER8") == 0)
	{
	     ghdr.user8 = *value;
	      return;
	}
	else if(strcmp(thelabel,"USER9") == 0)
	{
	     ghdr.user9 = *value;
	      return;
	}
	else
	{
	       printf("Warning %s not matched in setfhv\n",thelabel);
	       return;
	}	

}

void getnhv_(char *label, int *value, int *access_error, long slen)
{	
	char token[16];
	char thelabel[16];
	int i;
	
	strncpy(token,label,(size_t) slen);
	/* convert to upper case */
	for(i=0;i<slen;i++)
	{
	    thelabel[i] = toupper((int) token[i]);
	}
	thelabel[i] = '\0';
	
	/*printf("token = %s thelabel = %s",token,thelabel);*/
	
	if(strcmp(thelabel,"NZYEAR") == 0)
	{
	     *value = ghdr.nzyear;
	      return;
	}
	else if(strcmp(thelabel,"NZJDAY") == 0)
	{
	     *value = ghdr.nzjday;
	      return;
	}
	else if(strcmp(thelabel,"NZHOUR") == 0)
	{
	     *value = ghdr.nzhour;
	      return;
	}
	else if(strcmp(thelabel,"NZMIN") == 0)
	{
	     *value = ghdr.nzmin;
	      return;
	}
	else if(strcmp(thelabel,"NZSEC") == 0)
	{
	     *value = ghdr.nzsec;
	      return;
	}
	else if(strcmp(thelabel,"NZMSEC") == 0)
	{
	     *value = ghdr.nzmsec;
	      return;
	}
	else if(strcmp(thelabel,"NVHDR") == 0)
	{
	     *value = ghdr.nvhdr;
	      return;
	}
	else if(strcmp(thelabel,"NPTS") == 0)
	{
	     *value = ghdr.npts;
	      return;
	}
	else
	{
	       printf("Warning %s not matched in getnhv\n",thelabel);
	       *value = -12345;
	       return;
	}	

}


void setnhv_(char *label, int *value, int *access_error, long slen)
{	
	char token[16];
	char thelabel[16];
	int i;
	
	thelabel[0] = '\0';
	   token[0] = '\0';

	strncpy(token,label,(size_t) slen);
	/* convert to upper case */
	for(i=0;i<slen;i++)
	{
	    thelabel[i] = toupper((int) token[i]);
	}
	thelabel[i] = '\0';
	
	/* printf("token = %s thelabel = %s",token,thelabel);*/
	
	if(strcmp(thelabel,"NZYEAR") == 0)
	{
	     ghdr.nzyear = *value;
	      return;
	}
	else if(strcmp(thelabel,"NZJDAY") == 0)
	{
	     ghdr.nzjday = *value;
	      return;
	}
	else if(strcmp(thelabel,"NZHOUR") == 0)
	{
	     ghdr.nzhour = *value;
	      return;
	}
	else if(strcmp(thelabel,"NZMIN") == 0)
	{
	     ghdr.nzmin = *value;
	      return;
	}
	else if(strcmp(thelabel,"NZSEC") == 0)
	{
	     ghdr.nzsec = *value;
	      return;
	}
	else if(strcmp(thelabel,"NZMSEC") == 0)
	{
	     ghdr.nzmsec = *value;
	      return;
	}
	else if(strcmp(thelabel,"NVHDR") == 0)
	{
	     ghdr.nvhdr = *value;
	      return;
	}
	else if(strcmp(thelabel,"NPTS") == 0)
	{
	     printf("NPTS = %d\n",*value);
	     ghdr.npts = *value;
	      return;
	}
	else
	{
	       printf("Warning: %s not found in setnhv\n",thelabel);
	       return;
	}	

}


void setihv_(char *label, char *value, int *access_error, long slen, long vlen)
{	
	char token[16], token1[16];
	char thelabel[16], thevalue[16];
	int i;
	
	thelabel[0] = '\0';
	   token[0] = '\0';

	strncpy(token,label,(size_t) slen);
	/* convert to upper case */
	for(i=0;i<slen;i++)
	{
	    thelabel[i] = toupper((int) token[i]);
	}
	thelabel[i] = '\0';
	
	strncpy(token1,value,(size_t) vlen);
	/* convert to upper case */
	for(i=0;i<vlen;i++)
	{
	    thevalue[i] = toupper((int) token1[i]);
	}
	thevalue[i] = '\0';

	/*printf("token = %s thelabel = %s",token,thelabel);*/
	
	if(strcmp(thelabel,"IFTYPE") == 0)
	{
	     if(strcmp(thevalue,"ITIME") == 0) 
	     	ghdr.iftype = 1;
	     return;
	}
	else
	{
	       printf("Warning: %s not found in setihv\n",thelabel);
	       return;
	}	

}



void setlhv_(char *label, int *value, int *access_error, long slen)
{	
	char token[16];
	char thelabel[16];
	int i;
	
	thelabel[0] = '\0';
	   token[0] = '\0';

	strncpy(token,label,(size_t) slen);
	/* convert to upper case */
	for(i=0;i<slen;i++)
	{
	    thelabel[i] = toupper((int) token[i]);
	}
	thelabel[i] = '\0';
	
	/*printf("token = %s thelabel = %s",token,thelabel);*/
	
	if(strcmp(thelabel,"LEVEN") == 0)
	{
	     ghdr.leven = *value;
	     return;
	}
	else
	{
	       printf("Warning: %s not found in setlhv\n",thelabel);
	       return;
	}	

}

/*****************************************************************************/
/* write a sac spectral file ??? */
/*
void 
wrtsac1(FO, dt, ns, ar)
	char            FO[];
	int             ns;
	float           dt, ar[];
{
	FILE           *strm;
	struct SacHeader      hd;
	void            NewSacHeader();
	int             i;
	float           zero;
	zero = 0.;
	NewSacHeader(&hd);
	hd.npts = ns;
	hd.iftype = IRLIM;
	hd.iztype = IB;
	hd.b = zero;
	hd.e = 0.5 / dt;
	hd.internal2 = 0.;
	hd.internal3 = dt;
	hd.internal7 = ns;
	ns = ns / 2;
	hd.delta = hd.e / ns;
	hd.leven = TRUE;
	strm = fopen(FO, "w");
	fwrite(&hd, sizeof(struct SacHeader), 1, strm);
	for (i = 0; i < ns; i++)
		fwrite(&ar[2 * i], sizeof(float), 1, strm);
	fwrite(&ar[1], sizeof(float), 1, strm);
	for (i = ns - 1; i > 0; i--)
		fwrite(&ar[2 * i], sizeof(float), 1, strm);
	fwrite(&zero, sizeof(float), 1, strm);
	for (i = 1; i < ns; i++)
		fwrite(&ar[2 * i + 1], sizeof(float), 1, strm);
	fwrite(&zero, sizeof(float), 1, strm);
	for (i = ns - 1; i > 0; i--) {
		zero = -ar[2 * i + 1];
		fwrite(&zero, sizeof(float), 1, strm);
	}
	fclose(strm);
}

/*****************************************************************************/
/* Write an IMAGE sac file ??? */
/*
void 
wrtsac3(FO, nx, ny, xmin, xmax, ymin, ymax, ar)
	char            FO[];
	int             nx, ny;
	float           xmin, xmax, ymin, ymax, ar[];
{
	FILE           *strm;
	struct SacHeader      hd;
	void            NewSacHeader();
	NewSacHeader(&hd);
	hd.npts = nx * ny;
	hd.iftype = INIV51;
	hd.delta = 1.;
	hd.iztype = IB;
	hd.b = 0.;
	hd.e = hd.npts * hd.delta;
	hd.leven = TRUE;
	hd.unused2 = xmin;
	hd.unused3 = xmax;
	hd.unused4 = ymin;
	hd.unused5 = ymax;
	hd.unused13 = nx;
	hd.unused14 = ny;
	strm = fopen(FO, "w");
	fwrite(&hd, sizeof(struct SacHeader), 1, strm);
	fwrite(ar, hd.npts * sizeof(float), 1, strm);
	fclose(strm);
}
/*****************************************************************************/


/* compute the length of a space-terminated fortran string with
   dimensions slen. The return value is less than or equal to slen.
*/
   
int fslen(char *s,long slen)
{
	int i;

	i = 0;
	while(i < slen)
	  {
            /*if(isblank(s[i])) break;*/
            if(isspace(s[i])) break;
            i++;
	  }
		
	return(i);
	
}
int fslen_(char *s,long slen)
{
                
        return(fslen(s,slen));
        
}


/* function to convert a fortran string with dimension slen 
   into a null-terminated string length. Only the part of the
   string before the first space is copied.
*/

void fscpy(char *cc, char *fs, int slen)
{
	int ilen;
	ilen = fslen(fs,slen);
	strncpy(cc,fs,ilen);
	cc[ilen] = '\0';
}

/* 
    function to clear a fortran string and make is all spaces
        with a null termination at the end.
        
    fortran call: call fclearstring(myString)
    
    Another effort to win the battle with fortran strings...
    
*/

void fclearstring_(char *s, int slen)
{
	int i;
	for(i=0; i<slen; i++)
		s[i] = ' ';
		
	s[slen] = '\0';

}
